import glob
import json
import os

from injector import inject, singleton

from .store import StoreClient
from . import config


@singleton
class DataLog(object):
    @inject
    def __init__(self, configuration: config.Config, store_client: StoreClient):
        self.log_path = configuration.log_path
        self.store_client = store_client

    def store(self, data, timestamp):
        with open(os.path.join(self.log_path, timestamp + '.json'), 'w') as output_file:
            output_file.write(json.dumps(data))

    def transmit_stored_data(self):
        data_file_names = glob.glob(os.path.join(self.log_path, '*.json'))
        data_file_names.sort()

        for data_file_name in data_file_names:
            with open(data_file_name, 'r') as input_file:
                try:
                    data = json.loads(input_file.read())
                except:
                    print("error loading data {}".format(data_file_name))
                    continue
                print("{}: {}".format(data_file_name, data))
                try:
                    print("client:", self.store_client)
                    self.store_client.store(data)
                except Exception as e:
                    print("transmission error of archive - skipping", e)
                    break
            os.unlink(data_file_name)



# -*- coding: utf8 -*-

from injector import singleton, inject

try:
    import configparser
except ImportError:
    import configparser as configparser

from sht1x.Sht1x import Sht1x as SHT1x


@singleton
class Sensor:
    name = "SHT1x"

    @inject
    def __init__(self, config_parser: configparser.ConfigParser):
        data_pin = int(config_parser.get('sht1x_sensor', 'data_pin'))
        sck_pin = int(config_parser.get('sht1x_sensor', 'sck_pin'))

        self.sht1x = SHT1x(dataPin=data_pin, sckPin=sck_pin, gpioMode=SHT1x.GPIO_BCM)

    def measure(self, data_builder):
        (temperature, humidity) = self.sht1x.read_temperature_C_and_humidity()

        if temperature > -40.0:
            print("valid values")
            try:
                dew_point = self.sht1x.calculate_dew_point(temperature, humidity)
                dew_point = round(dew_point, 2)
            except ValueError:
                dew_point = None

            temperature = round(temperature, 2)
            humidity = round(humidity, 2)
        else:
            temperature = None
            humidity = None
            dew_point = None

        data_builder.add(self.name, "temperature", "°C", temperature)
        if dew_point:
            data_builder.add(self.name, "dew point", "°C", dew_point, True)
        data_builder.add(self.name, "relative humidity", "%", humidity)



import json

import pika
from injector import inject

from .. import config
from .client import StoreClient


class QueueStore(StoreClient):
    @inject
    def __init__(self, configuration : config.Config):
        try:
            credentials = pika.PlainCredentials(configuration.queue_username, configuration.queue_password)
            self.connection = pika.BlockingConnection(pika.ConnectionParameters(
                host=configuration.queue_host,
                port=configuration.queue_port,
                virtual_host=configuration.queue_virtual_host,
                credentials=credentials
            ))
        except Exception as e:
            print("could not create client", e)
            self.connection = None

    def __del__(self):
        if self.connection:
            self.connection.close()

    def store(self, data: dict):
        if self.connection:

            print("write data")
            channel = self.connection.channel()
            channel.confirm_delivery()

            delivery_confirmed = channel.basic_publish(exchange='', routing_key='measurement', body=json.dumps(data),
                                                       mandatory=True)

            if not delivery_confirmed:
                print("message was not confirmed")
                raise RuntimeError("message was not confirmed")

        else:
            print("client not available")
            raise RuntimeError("bla")



import socket

from assertpy import assert_that
from mock import Mock, PropertyMock, call

import klimalogger


class TestDataBuilderTest(object):
    def setup(self):
        self.config_parser = Mock()
        self.uut = klimalogger.config.Config(self.config_parser)

    def test_location_name(self):
        self.config_parser.get.return_value = '<location>'

        result = self.uut.client_location_name

        self.config_parser.get.assert_called_once_with('client', 'location_name')
        assert_that(result).is_equal_to('<location>')

    def test_client_host_name(self):
        result = self.uut.client_host_name

        self.config_parser.assert_not_called()
        assert_that(result).is_equal_to(socket.gethostname())

    def test_username(self):
        self.config_parser.get.return_value = '<username>'

        result = self.uut.store_username

        self.config_parser.get.assert_called_once_with('store', 'username')
        assert_that(result).is_equal_to('<username>')

    def test_password(self):
        self.config_parser.get.return_value = '<password>'

        result = self.uut.store_password

        self.config_parser.get.assert_called_once_with('store', 'password')
        assert_that(result).is_equal_to('<password>')

    def test_store_name(self):
        self.config_parser.get.return_value = '<store_name>'

        result = self.uut.store_name

        self.config_parser.get.assert_called_once_with('store', 'name')
        assert_that(result).is_equal_to('<store_name>')

    def test_store_host(self):
        self.config_parser.get.return_value = '<store_host>'

        result = self.uut.store_host

        self.config_parser.get.assert_called_once_with('store', 'host')
        assert_that(result).is_equal_to('<store_host>')

    def test_store_port(self):
        self.config_parser.get.return_value = '5'
        result = self.uut.store_port

        self.config_parser.get.assert_called_once_with('store', 'port')
        assert_that(result).is_equal_to(5)

    def test_log_path(self):
        self.config_parser.get.return_value = '<log_path>'
        result = self.uut.log_path

        self.config_parser.get.assert_called_once_with('log', 'path')
        assert_that(result).is_equal_to('<log_path>')



#!/usr/bin/env python3
# -*- coding: utf8 -*-

import pika

credentials = pika.PlainCredentials('klima', 'klima')
connection = pika.BlockingConnection(pika.ConnectionParameters('queue', credentials=credentials, virtual_host='klima'))

channel = connection.channel()

channel.confirm_delivery()
properties = pika.BasicProperties(content_type='application/json', content_encoding="utf-8",
                                  type='measurement')
publish = channel.basic_publish(exchange='', routing_key='measurement', body='{"content": "Hello World!"}', properties=properties)
print(" [x] Sent 'Hello World!'")

connection.close()



import os
import socket

from injector import singleton, provider, Module, inject, ClassProvider
from lazy import lazy

import configparser


@singleton
class Config(object):
    @inject
    def __init__(self, config_parser: configparser.ConfigParser):
        self.config_parser = config_parser

    @lazy
    def client_location_name(self) -> str:
        return self.config_parser.get('client', 'location_name')

    @lazy
    def client_host_name(self) -> str:
        return socket.gethostname()

    @lazy
    def store_type(self) -> str:
        return self.config_parser.get('store', 'type', fallback='direct')

    @lazy
    def store_username(self) -> str:
        return self.config_parser.get('store', 'username')

    @lazy
    def store_password(self) -> str:
        return self.config_parser.get('store', 'password')

    @lazy
    def store_name(self) -> str:
        return self.config_parser.get('store', 'name')

    @lazy
    def store_host(self) -> str:
        return self.config_parser.get('store', 'host')

    @lazy
    def store_port(self) -> int:
        port_string = self.config_parser.get('store', 'port')
        return int(port_string)

    @lazy
    def queue_username(self) -> str:
        return self.config_parser.get('queue', 'username')

    @lazy
    def queue_password(self) -> str:
        return self.config_parser.get('queue', 'password')

    @lazy
    def queue_host(self) -> str:
        return self.config_parser.get('queue', 'host')

    @lazy
    def queue_port(self) -> int:
        port_string = self.config_parser.get('queue', 'port')
        return int(port_string)

    @lazy
    def queue_virtual_host(self) -> str:
        return self.config_parser.get('queue', 'virtual_host')

    @lazy
    def log_path(self) -> str:
        return self.config_parser.get('log', 'path')


class ConfigModule(Module):
    @singleton
    @provider
    def provide_config_parser(self) -> configparser.ConfigParser:

        config_file_locations = ['klimalogger.conf', '/etc/klimalogger/klimalogger.conf', '/etc/klimalogger.conf']

        for config_file_location in config_file_locations:
            if os.path.exists(config_file_location):
                print("reading config file location", config_file_location)
                config_parser = configparser.ConfigParser()
                config_parser.read(config_file_location)
                return config_parser

        raise IOError("config file not found")



from injector import singleton, inject, Injector

from .store import StoreClient
from .config import ConfigModule
from .data_builder import DataBuilder
from .data_log import DataLog
from .sensor_factory import SensorFactory
from .store import StoreModule

INJECTOR = Injector(
    [StoreModule(), ConfigModule()])


@singleton
class Client(object):
    @inject
    def __init__(self, data_builder:DataBuilder, sensor_factory: SensorFactory, store_client: StoreClient, data_log: DataLog):
        self.data_builder = data_builder
        self.sensor_factory = sensor_factory
        self.store_client = store_client
        self.data_log = data_log

    def measure_and_store(self):
        (timestamp, data) = self.measure()

        try:
            self.store_client.store(data)
            print("stored data")
        except Exception as e:
            print("error during data transmission: create local log entry", e)
            self.data_log.store(data, timestamp)

        self.data_log.transmit_stored_data()

    def measure(self):
        self.sensor_factory.measure(self.data_builder)

        return (self.data_builder.timestamp, self.data_builder.data)


def client():
    return INJECTOR.get(Client)



#!/usr/bin/env python3
# -*- coding: utf8 -*-
import json

import pika

credentials = pika.PlainCredentials('klima', 'klima')
connection = pika.BlockingConnection(pika.ConnectionParameters('queue', credentials=credentials, virtual_host='klima'))

channel = connection.channel()

channel.queue_declare(queue='measurement', durable=True)

def callback(ch, method, properties, body):
    #import pdb;
    #pdb.set_trace()


    content_encoding = properties.content_encoding
    decoded_body = body.decode(content_encoding)

    content_type = properties.content_type
    if content_type == 'application/json':
        decoded_body = json.loads(decoded_body)

    print(" [x] Received %r" % (decoded_body))

channel.basic_consume(callback,
                      queue='measurement',
                      no_ack=True)

print(' [*] Waiting for messages. To exit press CTRL+C')
channel.start_consuming()




import logging

from injector import singleton, inject, Injector

import importlib

import configparser

from .data_builder import DataBuilder

log = logging.getLogger(__name__)

@singleton
class SensorFactory(object):
    @inject
    def __init__(self, configuration: configparser.ConfigParser, current_injector : Injector):
        self.sensors = [sensor.strip() for sensor in configuration.get('client', 'sensors').split(',')]
        self.current_injector = current_injector

    def measure(self, data_builder: DataBuilder):
        for sensor in self.sensors:
            try:
                module = importlib.import_module('klimalogger.sensor.' + sensor + '_sensor')
                log.info("sensor: %s, module: %s", sensor, module)
                sensor = self.current_injector.get(module.Sensor)
                sensor.measure(data_builder)
                del(sensor)
            except BaseException as e:
                log.error("measurement of sensor %s failed", sensor, e)




# -*- coding: utf8 -*-

from injector import singleton, inject

try:
    import configparser
except ImportError:
    import configparser as configparser

import board
import adafruit_bme680

@singleton
class Sensor:
    name = "BME680"

    @inject
    def __init__(self, config_parser: configparser.ConfigParser):
        self.elevation = int(config_parser.get('bme680_sensor', 'elevation'))
        i2c = board.I2C()
        self.sensor = adafruit_bme680.Adafruit_BME680_I2C(i2c)

    def measure(self, data_builder):
        temperature = self.sensor.temperature
        relative_humidity = self.sensor.relative_humidity
        pressure = self.sensor.pressure
        voc_gas = self.sensor.gas

        data_builder.add(self.name, "temperature", "°C", temperature)
        data_builder.add(self.name, "relative humidity", "%", relative_humidity)
        data_builder.add(self.name, "pressure", "Pa", pressure)
        data_builder.add(self.name, "voc gas", "Ohm", voc_gas)


# -*- coding: utf8 -*-



from injector import singleton, inject

try:
    import configparser
except ImportError:
    import configparser as configparser

from Adafruit_BMP.BMP085 import BMP085


@singleton
class Sensor(object):
    name = "BMP085"

    @inject
    def __init__(self, config_parser : configparser.ConfigParser):
        self.elevation = int(config_parser.get('bmp085_sensor', 'elevation'))

        self.bmp085 = BMP085()

    def measure(self, data_builder):
        pressure = round(self.bmp085.read_sealevel_pressure(self.elevation) / 100, 2)

        data_builder.add("BMP085", "atmospheric pressure", "hPa", pressure)



# -*- coding: utf8 -*-

from injector import singleton, inject

try:
    import configparser
except ImportError:
    import configparser as configparser

import board
import busio
import adafruit_sgp30


@singleton
class Sensor:
    name = "SGP30"

    @inject
    def __init__(self, config_parser: configparser.ConfigParser):
        i2c_bus = busio.I2C(board.SCL, board.SDA, frequency=100000)
        self.sensor = adafruit_sgp30.Adafruit_SGP30(i2c_bus)

    def measure(self, data_builder):
        eCO2, TVOC = self.sensor.iaq_measure()

        data_builder.add(self.name, "eCO2", "ppm", eCO2)
        data_builder.add(self.name, "TVOC", "ppb", TVOC)



#!/usr/bin/env python3
# -*- coding: utf8 -*-

from setuptools import setup, find_packages

import glob

setup(
    name='klimalogger',
    version='0.4.3',
    packages=find_packages(exclude=('tests')),
    scripts=glob.glob('scripts/*'),
    url='https://github.com/wuan/klimalogger',
    license='License :: OSI Approved :: Apache License 2.0',
    author='Andreas Würl',
    author_email='andreas@wuerl.net',
    description='Simple data logging client for InfluxDB',
    classifiers=[ 
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: Apache Software License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    install_requires=['influxdb', 'injector', 'lazy'],
    tests_require=['pytest-cov', 'mock', 'assertpy']
)



from injector import singleton, inject

from influxdb import InfluxDBClient

from ..config import Config
from .client import StoreClient

class InfluxDbStore(StoreClient):
    @inject
    def __init__(self, configuration: Config):
        try:
            self.client = InfluxDBClient(
                host=configuration.store_host,
                port=configuration.store_port,
                database=configuration.store_name,
                username=configuration.store_username,
                password=configuration.store_password,
                timeout=5)
        except Exception as e:
            print("could not create client", e)
            self.client = None

    def store(self, data: dict):
        if self.client:
            print("write data")
            self.client.write_points(data)
        else:
            print("client not available")
            raise RuntimeError("client not available")



from abc import ABCMeta, abstractmethod

from injector import Module, provider, singleton

from ..config import Config
from .client import StoreClient


class StoreModule(Module):
    @provider
    @singleton
    def store_provider(self, config: Config) -> StoreClient:
        if config.store_type == 'queue':
            from .queue import QueueStore
            return QueueStore(config)
        else:
            from .influxdb import InfluxDbStore
            return InfluxDbStore(config)
